/**
 * Copyright: Copyright (c) 2009 Jacob Carlborg.
 * Authors: Jacob Carlborg
 * Version: Initial created: Aug 3, 2009 
 * License: $(LINK2 http://www.boost.org/LICENSE_1_0.txt, Boost Software License 1.0)
 */
module dstep.foundation.NSCompoundPredicate;

import dstep.foundation.NSArray;
import dstep.foundation.NSObjCRuntime;
import dstep.foundation.NSPredicate;
import dstep.objc.bridge.Bridge;
import dstep.objc.objc;

alias NSUInteger NSCompoundPredicateType;

enum
{
	NSNotPredicateType = 0,
	NSAndPredicateType,
	NSOrPredicateType
}

class NSCompoundPredicate : NSPredicate
{
	mixin (ObjcWrap);
	
	this ()
	{
		super(typeof(this).alloc.init.objcObject);
	}
	
	typeof(this) init ()
	{
		return invokeObjcSelf!(typeof(this), "init");
	}

	Object initWithType (uint type, NSArray subpredicates)
	{
		return invokeObjcSelf!(Object, "initWithType:subpredicates:", uint, NSArray)(type, subpredicates);
	}

	this (uint type, NSArray subpredicates)
	{
		typeof(this).alloc.initWithType(type, subpredicates);
	}

	uint compoundPredicateType ()
	{
		return invokeObjcSelf!(uint, "compoundPredicateType");
	}

	NSArray subpredicates ()
	{
		return invokeObjcSelf!(NSArray, "subpredicates");
	}

	static NSPredicate andPredicateWithSubpredicates (NSArray subpredicates)
	{
		return invokeObjcSuperClass!(NSPredicate, "andPredicateWithSubpredicates:", NSArray)(subpredicates);
	}

	static NSPredicate orPredicateWithSubpredicates (NSArray subpredicates)
	{
		return invokeObjcSuperClass!(NSPredicate, "orPredicateWithSubpredicates:", NSArray)(subpredicates);
	}

	static NSPredicate notPredicateWithSubpredicate (NSPredicate predicate)
	{
		return invokeObjcSuperClass!(NSPredicate, "notPredicateWithSubpredicate:", NSPredicate)(predicate);
	}
}

