/**
 * Copyright: Copyright (c) 2009 Jacob Carlborg.
 * Authors: Jacob Carlborg
 * Version: Initial created: Aug 3, 2009 
 * License: $(LINK2 http://www.boost.org/LICENSE_1_0.txt, Boost Software License 1.0)
 */
module dstep.foundation.NSOperation;

import dstep.foundation.NSArray;
import dstep.foundation.NSInvocation;
import dstep.foundation.NSObjCRuntime;
import dstep.foundation.NSObject;
import dstep.foundation.NSSet;
import dstep.foundation.NSString;
import dstep.objc.bridge.Bridge;
import dstep.objc.objc;



import bindings = dstep.foundation.NSOperation_bindings;

alias NSInteger NSOperationQueuePriority;

private
{
	NSString NSInvocationOperationVoidResultException_;
	NSString NSInvocationOperationCancelledException_;
}

NSString NSInvocationOperationVoidResultException ()
{
	if (NSInvocationOperationVoidResultException_)
		return NSInvocationOperationVoidResultException_;
		
	return NSInvocationOperationVoidResultException_ = new NSString(bindings.NSInvocationOperationVoidResultException);
}

NSString NSInvocationOperationCancelledException ()
{
	if (NSInvocationOperationCancelledException_)
		return NSInvocationOperationCancelledException_;
		
	return NSInvocationOperationCancelledException_ = new NSString(bindings.NSInvocationOperationCancelledException);
}

enum
{
	NSOperationQueuePriorityVeryLow = -8,
	NSOperationQueuePriorityLow = -4,
	NSOperationQueuePriorityNormal = 0,
	NSOperationQueuePriorityHigh = 4,
	NSOperationQueuePriorityVeryHigh = 8
}

enum
{
	NSOperationQueueDefaultMaxConcurrentOperationCount = -1
}

class NSOperation : NSObject
{
	mixin (ObjcWrap);
	
	this ()
	{
		super(typeof(this).alloc.init.objcObject);
	}
	
	typeof(this) init ()
	{
		return invokeObjcSelf!(typeof(this), "init");
	}

	void start ()
	{
		return invokeObjcSelf!(void, "start");
	}

	void main ()
	{
		return invokeObjcSelf!(void, "main");
	}

	bool isCancelled ()
	{
		return invokeObjcSelf!(bool, "isCancelled");
	}

	void cancel ()
	{
		return invokeObjcSelf!(void, "cancel");
	}

	bool isExecuting ()
	{
		return invokeObjcSelf!(bool, "isExecuting");
	}

	bool isFinished ()
	{
		return invokeObjcSelf!(bool, "isFinished");
	}

	bool isConcurrent ()
	{
		return invokeObjcSelf!(bool, "isConcurrent");
	}

	bool isReady ()
	{
		return invokeObjcSelf!(bool, "isReady");
	}

	void addDependency (NSOperation op)
	{
		return invokeObjcSelf!(void, "addDependency:", NSOperation)(op);
	}

	void removeDependency (NSOperation op)
	{
		return invokeObjcSelf!(void, "removeDependency:", NSOperation)(op);
	}

	NSArray dependencies ()
	{
		return invokeObjcSelf!(NSArray, "dependencies");
	}

	int queuePriority ()
	{
		return invokeObjcSelf!(int, "queuePriority");
	}

	void setQueuePriority (int p)
	{
		return invokeObjcSelf!(void, "setQueuePriority:", int)(p);
	}
}

class NSOperationQueue : NSObject
{
	mixin (ObjcWrap);
	
	this ()
	{
		super(typeof(this).alloc.init.objcObject);
	}
	
	typeof(this) init ()
	{
		return invokeObjcSelf!(typeof(this), "init");
	}

	void addOperation (NSOperation op)
	{
		return invokeObjcSelf!(void, "addOperation:", NSOperation)(op);
	}

	NSArray operations ()
	{
		return invokeObjcSelf!(NSArray, "operations");
	}

	NSInteger maxConcurrentOperationCount ()
	{
		return invokeObjcSelf!(NSInteger, "maxConcurrentOperationCount");
	}

	void setMaxConcurrentOperationCount (NSInteger cnt)
	{
		return invokeObjcSelf!(void, "setMaxConcurrentOperationCount:", NSInteger)(cnt);
	}

	void setSuspended (bool b)
	{
		return invokeObjcSelf!(void, "setSuspended:", bool)(b);
	}

	bool isSuspended ()
	{
		return invokeObjcSelf!(bool, "isSuspended");
	}

	void cancelAllOperations ()
	{
		return invokeObjcSelf!(void, "cancelAllOperations");
	}

	void waitUntilAllOperationsAreFinished ()
	{
		return invokeObjcSelf!(void, "waitUntilAllOperationsAreFinished");
	}
}

class NSInvocationOperation : NSOperation
{
	mixin (ObjcWrap);
	
	this ()
	{
		super(typeof(this).alloc.init.objcObject);
	}
	
	typeof(this) init ()
	{
		return invokeObjcSelf!(typeof(this), "init");
	}

	Object initWithTarget (Object target, SEL sel, Object arg)
	{
		return invokeObjcSelf!(Object, "initWithTarget:selector:object:", Object, SEL, Object)(target, sel, arg);
	}

	this (Object target, SEL sel, Object arg)
	{
		typeof(this).alloc.initWithTarget(target, sel, arg);
	}

	Object initWithInvocation (NSInvocation inv)
	{
		return invokeObjcSelf!(Object, "initWithInvocation:", NSInvocation)(inv);
	}

	this (NSInvocation inv)
	{
		typeof(this).alloc.initWithInvocation(inv);
	}

	NSInvocation invocation ()
	{
		return invokeObjcSelf!(NSInvocation, "invocation");
	}

	Object result ()
	{
		return invokeObjcSelf!(Object, "result");
	}
}

