/**
 * Copyright: Copyright (c) 2009 Jacob Carlborg.
 * Authors: Jacob Carlborg
 * Version: Initial created: Aug 3, 2009 
 * License: $(LINK2 http://www.boost.org/LICENSE_1_0.txt, Boost Software License 1.0)
 */
module dstep.foundation.NSXMLDocument;

import dstep.foundation.NSArray;
import dstep.foundation.NSData;
import dstep.foundation.NSDictionary;
import dstep.foundation.NSError;
import dstep.foundation.NSObjCRuntime;
import dstep.foundation.NSString;
import dstep.foundation.NSURL;
import dstep.foundation.NSXMLDTD;
import dstep.foundation.NSXMLElement;
import dstep.foundation.NSXMLNode;
import dstep.objc.bridge.Bridge;
import dstep.objc.objc;

alias NSUInteger NSXMLDocumentContentKind;

enum
{
	NSXMLDocumentXMLKind = 0,
	NSXMLDocumentXHTMLKind,
	NSXMLDocumentHTMLKind,
	NSXMLDocumentTextKind
}

class NSXMLDocument : NSXMLNode
{
	mixin (ObjcWrap);
	
	this ()
	{
		super(typeof(this).alloc.init.objcObject);
	}
	
	typeof(this) init ()
	{
		return invokeObjcSelf!(typeof(this), "init");
	}

	Object initWithXMLString (NSString string, NSUInteger mask, ref NSError error)
	{
		id err;
		
		if (error)
			err = new objc_object;
				
		Object result = invokeObjcSelf!(Object, "initWithXMLString:options:error:", NSString, NSUInteger, id*)(string, mask, &err);
		
		if (err)
			error = new NSError(err);
		
		return result;
	}

	this (NSString string, NSUInteger mask, ref NSError error)
	{
		typeof(this).alloc.initWithXMLString(string, mask, error);
	}

	Object initWithContentsOfURL (NSURL url, NSUInteger mask, ref NSError error)
	{
		id err;
		
		if (error)
			err = new objc_object;
		
		Object result = invokeObjcSelf!(Object, "initWithContentsOfURL:options:error:", NSURL, NSUInteger, id*)(url, mask, &err);
		
		if (err)
			error = new NSError(err);
		
		return result;
	}

	this (NSURL url, NSUInteger mask, ref NSError error)
	{
		typeof(this).alloc.initWithContentsOfURL(url, mask, error);
	}

	Object initWithData (NSData data, NSUInteger mask, ref NSError error)
	{
		id err;
		
		if (error)
			err = new objc_object;
		
		Object result = invokeObjcSelf!(Object, "initWithData:options:error:", NSData, NSUInteger, id*)(data, mask, &err);
		
		if (err)
			error = new NSError(err);
		
		return result;
	}

	this (NSData data, NSUInteger mask, ref NSError error)
	{
		typeof(this).alloc.initWithData(data, mask, error);
	}

	Object initWithRootElement (NSXMLElement element)
	{
		return invokeObjcSelf!(Object, "initWithRootElement:", NSXMLElement)(element);
	}

	this (NSXMLElement element)
	{
		typeof(this).alloc.initWithRootElement(element);
	}

	static Class replacementClassForClass (Class cls)
	{
		return invokeObjcSuperClass!(Class, "replacementClassForClass:", Class)(cls);
	}

	void setCharacterEncoding (NSString encoding)
	{
		return invokeObjcSelf!(void, "setCharacterEncoding:", NSString)(encoding);
	}

	NSString characterEncoding ()
	{
		return invokeObjcSelf!(NSString, "characterEncoding");
	}

	void setVersion (NSString version_)
	{
		return invokeObjcSelf!(void, "setVersion:", NSString)(version_);
	}

	NSString version_ ()
	{
		return invokeObjcSelf!(NSString, "version");
	}

	void setStandalone (bool standalone)
	{
		return invokeObjcSelf!(void, "setStandalone:", bool)(standalone);
	}

	bool isStandalone ()
	{
		return invokeObjcSelf!(bool, "isStandalone");
	}

	void setDocumentContentKind (uint kind)
	{
		return invokeObjcSelf!(void, "setDocumentContentKind:", uint)(kind);
	}

	uint documentContentKind ()
	{
		return invokeObjcSelf!(uint, "documentContentKind");
	}

	void setMIMEType (NSString MIMEType)
	{
		return invokeObjcSelf!(void, "setMIMEType:", NSString)(MIMEType);
	}

	NSString MIMEType ()
	{
		return invokeObjcSelf!(NSString, "MIMEType");
	}

	void setDTD (NSXMLDTD documentTypeDeclaration)
	{
		return invokeObjcSelf!(void, "setDTD:", NSXMLDTD)(documentTypeDeclaration);
	}

	NSXMLDTD DTD ()
	{
		return invokeObjcSelf!(NSXMLDTD, "DTD");
	}

	void setRootElement (NSXMLNode root)
	{
		return invokeObjcSelf!(void, "setRootElement:", NSXMLNode)(root);
	}

	NSXMLElement rootElement ()
	{
		return invokeObjcSelf!(NSXMLElement, "rootElement");
	}

	void insertChild (NSXMLNode child, NSUInteger index)
	{
		return invokeObjcSelf!(void, "insertChild:atIndex:", NSXMLNode, NSUInteger)(child, index);
	}

	void insertChildren (NSArray children, NSUInteger index)
	{
		return invokeObjcSelf!(void, "insertChildren:atIndex:", NSArray, NSUInteger)(children, index);
	}

	void removeChildAtIndex (NSUInteger index)
	{
		return invokeObjcSelf!(void, "removeChildAtIndex:", NSUInteger)(index);
	}

	void setChildren (NSArray children)
	{
		return invokeObjcSelf!(void, "setChildren:", NSArray)(children);
	}

	void addChild (NSXMLNode child)
	{
		return invokeObjcSelf!(void, "addChild:", NSXMLNode)(child);
	}

	void replaceChildAtIndex (NSUInteger index, NSXMLNode node)
	{
		return invokeObjcSelf!(void, "replaceChildAtIndex:withNode:", NSUInteger, NSXMLNode)(index, node);
	}

	NSData XMLData ()
	{
		return invokeObjcSelf!(NSData, "XMLData");
	}

	NSData XMLDataWithOptions (NSUInteger options)
	{
		return invokeObjcSelf!(NSData, "XMLDataWithOptions:", NSUInteger)(options);
	}

	Object objectByApplyingXSLT (NSData xslt, NSDictionary arguments, ref NSError error)
	{
		id err;
		
		if (error)
			err = new objc_object;
		
		Object result = invokeObjcSelf!(Object, "objectByApplyingXSLT:arguments:error:", NSData, NSDictionary, id*)(xslt, arguments, &err);
		
		if (err)
			error = new NSError(err);
		
		return result;
	}

	Object objectByApplyingXSLTString (NSString xslt, NSDictionary arguments, ref NSError error)
	{
		id err;
		
		if (error)
			err = new objc_object;
		
		Object result = invokeObjcSelf!(Object, "objectByApplyingXSLTString:arguments:error:", NSString, NSDictionary, id*)(xslt, arguments, &err);
		
		if (err)
			error = new NSError(err);
		
		return result;
	}

	Object objectByApplyingXSLTAtURL (NSURL xsltURL, NSDictionary argument, ref NSError error)
	{
		id err;
		
		if (error)
			err = new objc_object;
		
		Object result = invokeObjcSelf!(Object, "objectByApplyingXSLTAtURL:arguments:error:", NSURL, NSDictionary, id*)(xsltURL, argument, &err);
		
		if (err)
			error = new NSError(err);
		
		return result;
	}

	bool validateAndReturnError (ref NSError error)
	{
		id err;
		
		if (error)
			err = new objc_object;
		
		bool result = invokeObjcSelf!(bool, "validateAndReturnError:", id*)(&err);
		
		if (err)
			error = new NSError(err);
		
		return result;
	}
}

