/**
 * Copyright: Copyright (c) 2009 Jacob Carlborg.
 * Authors: Jacob Carlborg
 * Version: Initial created: Sep 24, 2009 
 * License: $(LINK2 http://www.boost.org/LICENSE_1_0.txt, Boost Software License 1.0)
 */
module dstep.quartzcore.CAMediaTimingFunction;

import dstep.foundation.NSArray;
import dstep.foundation.NSCoder;
import dstep.foundation.NSObject;
import dstep.foundation.NSString;
import dstep.objc.bridge.Bridge;
import dstep.objc.objc;
import dstep.quartzcore.CAMediaTiming;

import bindings = dstep.quartzcore.CAMediaTimingFunction_bindings;

private
{
	NSString kCAMediaTimingFunctionLinear_;
	NSString kCAMediaTimingFunctionEaseIn_;
	NSString kCAMediaTimingFunctionEaseOut_;
	NSString kCAMediaTimingFunctionEaseInEaseOut_;
}

NSString kCAMediaTimingFunctionLinear ()
{
	if (kCAMediaTimingFunctionLinear_)
		return kCAMediaTimingFunctionLinear_;

	return kCAMediaTimingFunctionLinear_ = new NSString(bindings.kCAMediaTimingFunctionLinear);
}

NSString kCAMediaTimingFunctionEaseIn ()
{
	if (kCAMediaTimingFunctionEaseIn_)
		return kCAMediaTimingFunctionEaseIn_;

	return kCAMediaTimingFunctionEaseIn_ = new NSString(bindings.kCAMediaTimingFunctionEaseIn);
}

NSString kCAMediaTimingFunctionEaseOut ()
{
	if (kCAMediaTimingFunctionEaseOut_)
		return kCAMediaTimingFunctionEaseOut_;

	return kCAMediaTimingFunctionEaseOut_ = new NSString(bindings.kCAMediaTimingFunctionEaseOut);
}

NSString kCAMediaTimingFunctionEaseInEaseOut ()
{
	if (kCAMediaTimingFunctionEaseInEaseOut_)
		return kCAMediaTimingFunctionEaseInEaseOut_;

	return kCAMediaTimingFunctionEaseInEaseOut_ = new NSString(bindings.kCAMediaTimingFunctionEaseInEaseOut);
}

class CAMediaTimingFunction : NSObject, INSCoding
{
	mixin (ObjcWrap);

	static Object functionWithName (NSString name)
	{
		return invokeObjcSelfClass!(Object, "functionWithName:", NSString)(name);
	}

	static Object functionWithControlPoints (float c1x, float c1y, float c2x, float c2y)
	{
		return invokeObjcSelfClass!(Object, "functionWithControlPoints::::", float, float, float, float)(c1x, c1y, c2x, c2y);
	}

	CAMediaTimingFunction initWithControlPoints (float c1x, float c1y, float c2x, float c2y)
	{
		id result = invokeObjcSelf!(id, "initWithControlPoints::::", float, float, float, float)(c1x, c1y, c2x, c2y);
		return result is this.objcObject ? this : (result !is null ? new CAMediaTimingFunction(result) : null);
	}

	this (float c1x, float c1y, float c2x, float c2y)
	{
		super(CAMediaTimingFunction.alloc.initWithControlPoints(c1x, c1y, c2x, c2y).objcObject);
	}

	/*void getControlPointAtIndex (size_t idx, float[2] ptr)
	{
		return invokeObjcSelf!(void, "getControlPointAtIndex:values:", size_t, float[2])(idx, ptr.ptr);
	}*/
	
	void encodeWithCoder (NSCoder aCoder)
	{
		return invokeObjcSelf!(void, "encodeWithCoder:", NSCoder)(aCoder);
	}
	
	CAMediaTimingFunction initWithCoder (NSCoder aDecoder)
	{
		return invokeObjcSelf!(CAMediaTimingFunction, "initWithCoder:", NSCoder)(aDecoder);
	}
}

